@@ -1,6 +1,6 @@
 # Deb-o-Matic
 #
-# Copyright (C) 2007-2013 Luca Falavigna
+# Copyright (C) 2007-2014 Luca Falavigna
 # Copyright (C) 2010 Alessio Treglia
 #
 # Author: Luca Falavigna <dktrkranz@debian.org>
@@ -21,28 +21,27 @@
 import os
 from ast import literal_eval
 from hashlib import sha256
-from lockfile import FileLock
-from re import findall, split
-from subprocess import call, PIPE
+from logging import debug, error, info
+from re import findall
+from subprocess import call, check_output
 from time import strftime
-from urllib2 import Request, urlopen, HTTPError, URLError
+from urllib.request import Request, urlopen
+from urllib.error import HTTPError, URLError
 
-from gpg import GPG
-from modules import Module
+from .gpg import GPG
+from .modules import Module
 
 
 class Build:
 
-    def __init__(self, opts, log, package=None, dsc=None,
-                 debopts=None, distribution=None, origin=None):
-        self.log = log
-        self.e = self.log.t
-        self.w = self.log.w
+    def __init__(self, opts, package=None, dsc=None, debopts=None,
+                 distribution=None, extrabd=None, origin=None):
         (self.opts, self.rtopts, self.conffile) = opts
         self.package = package
         self.dscfile = dsc
         self.debopts = debopts
         self.distribution = distribution
+        self.extrabd = extrabd
         self.origin = origin
         self.cmd = None
         self.distopts = {}
@@ -51,22 +50,19 @@ class Build:
         self.full = False
         self.uploader = None
 
-    def acquire_lock(self):
-        self.lockfile = FileLock('/var/run/debomatic.%s' %
-                                 (os.path.basename(self.configfile)))
-        self.lockfile.acquire()
-
     def build(self):
         if self.full:
-            self.w(_('Full build routine launched'), 3)
+            debug(_('Full build routine launched'))
         else:
-            self.w(_('Simple build routine launched'), 3)
+            debug(_('Simple build routine launched'))
         if self.dscfile:
             self.files.add(self.dscfile)
+            debug(_('File %s added') % self.dscfile)
         self.parse_distribution_options()
         if self.distribution in self.runtime_option('distblacklist'):
             self.remove_files()
-            self.e(_('Distribution %s is disabled' % self.distribution))
+            error(_('Distribution %s is disabled') % self.distribution)
+            raise RuntimeError
         self.fetch_missing_files()
         try:
             self.setup_pbuilder()
@@ -80,34 +76,40 @@ class Build:
         uploader_email = ''
         packageversion = os.path.splitext(os.path.basename(self.dscfile))[0]
         builddir = os.path.join(self.buildpath, 'pool', packageversion)
-        self.w(_('Building %s') % os.path.basename(self.dscfile))
+        info(_('Building %s') % os.path.basename(self.dscfile))
         if not os.path.exists(builddir):
             os.mkdir(builddir)
         if self.uploader:
-            uploader_email = self.uploader[1]
-        self.w(_('Pre-build hooks launched'), 3)
+            uploader_email = self.uploader[1].decode('utf-8')
+        builder = self.opts.get('default', 'builder')
+        architecture = self.opts.get('default', 'architecture')
+        if architecture == 'system':
+            b_arch = check_output(['dpkg-architecture', '-qDEB_BUILD_ARCH'])
+            architecture = b_arch.strip().decode('utf-8')
+        debug(_('Pre-build hooks launched'))
         mod.execute_hook('pre_build', {'cfg': self.configfile,
                                        'directory': self.buildpath,
                                        'distribution': self.distribution,
+                                       'architecture': architecture,
                                        'dsc': self.dscfile,
                                        'opts': self.opts,
                                        'package': packageversion,
                                        'uploader': uploader_email})
-        self.w(_('Pre-build hooks finished'), 3)
-        builder = self.opts.get('default', 'builder')
+        debug(_('Pre-build hooks finished'))
         if builder == 'cowbuilder':
             base = '--basepath'
         else:
             base = '--basetgz'
         debopts = ' '.join((self.get_build_options(),
-                            self.get_compression(),
                             self.get_changelog_versions(),
+                            self.get_compression(),
                             self.get_orig_tarball()))
         with open(os.devnull, 'w') as fd:
             try:
-                self.w(_('Launching %s') % builder , 2)
+                debug(_('Launching %s') % builder)
                 call([builder, '--build', '--override-config',
                      base, '%s/%s' % (self.buildpath, self.distribution),
+                     '--architecture', architecture,
                      '--logfile', '%s/pool/%s/%s.buildlog' %
                      (self.buildpath, packageversion, packageversion),
                      '--buildplace', '%s/build' % self.buildpath,
@@ -116,21 +118,23 @@ class Build:
                      '--aptcache', '%s/aptcache' % self.buildpath,
                      '--debbuildopts', '%s' % debopts,
                      '--hookdir', self.opts.get('default', 'pbuilderhooks'),
+                     '--extrapackages', '%s' % self.get_extra_builddeps(),
                      '--configfile', self.configfile, self.dscfile],
                      stdout=fd, stderr=fd)
             except OSError:
-                self.w(_('Unable to launch %s') % builder)
-        self.w(_('Post-build hooks launched'), 3)
+                error(_('Unable to launch %s') % builder)
+        debug(_('Post-build hooks launched'))
         mod.execute_hook('post_build', {'cfg': self.configfile,
                                         'directory': self.buildpath,
                                         'distribution': self.distribution,
+                                        'architecture': architecture,
                                         'dsc': self.dscfile,
                                         'opts': self.opts,
                                         'package': packageversion,
                                         'uploader': uploader_email})
-        self.w(_('Post-build hooks finished'), 3)
+        debug(_('Post-build hooks finished'))
         self.remove_files()
-        self.w(_('Build of %s complete') % os.path.basename(self.dscfile))
+        info(_('Build of %s complete') % os.path.basename(self.dscfile))
 
     def fetch_missing_files(self):
         filename = self.dscfile if self.dscfile else self.package
@@ -142,7 +146,8 @@ class Build:
                     break
         if not self.dscfile:
             self.remove_files()
-            self.e(_('Bad .changes file: %s') % self.package)
+            error(_('Bad .changes file: %s') % self.package)
+            raise RuntimeError
         with open(self.dscfile, 'r') as fd:
             data = fd.read()
         for entry in findall('\s\w{32}\s\d+\s(\S+)', data):
@@ -152,17 +157,20 @@ class Build:
                                       (self.distopts['origin'], component,
                                        findall('^lib\S|^\S', packagename)[0],
                                        packagename, entry))
+                    debug(_('Requesting URL %s') % request.get_full_url())
                     try:
-                        self.w(_('Downloading missing %s' % entry), 2)
+                        debug(_('Downloading missing %s') % entry)
                         data = urlopen(request).read()
                         break
                     except (HTTPError, URLError):
                         data = None
                 if data:
-                    with open(os.path.join(self.packagedir, entry), 'w') as e:
+                    with open(os.path.join(self.packagedir, entry), 'wb') as e:
                         e.write(data)
             if not (os.path.join(self.packagedir, entry)) in self.files:
-                self.files.add(os.path.join(self.packagedir, entry))
+                entry = os.path.join(self.packagedir, entry)
+                self.files.add(entry)
+                debug(_('File %s added') % entry)
 
     def get_build_options(self):
         if self.debopts:
@@ -188,12 +196,18 @@ class Build:
             if os.path.exists(pkgfile):
                 if findall('(.*\.debian\..*)', pkgfile):
                     try:
-                        compression = '-Z%s' % \
-                                       ext[os.path.splitext(pkgfile)[1]]
+                        compression = ('-Z%s' %
+                                       ext[os.path.splitext(pkgfile)[1]])
                     except IndexError:
                         pass
         return compression
 
+    def get_extra_builddeps(self):
+        if self.extrabd:
+            return self.extrabd
+        else:
+            return ''
+
     def get_orig_tarball(self):
         sa = ''
         if self.full:
@@ -215,32 +229,32 @@ class Build:
             else:
                 if isinstance(mapper, dict):
                     if self.distribution in mapper:
-                        self.w(_('%(mapped)s mapped as %(mapper)s') % 
-                               {'mapped': self.distribution,
-                               'mapper': mapper[self.distribution]}, 3)
+                        debug(_('%(mapped)s mapped as %(mapper)s') %
+                              {'mapped': self.distribution,
+                               'mapper': mapper[self.distribution]})
                         self.distribution = mapper[self.distribution]
 
     def needs_update(self):
+        distribution = self.distopts['distribution']
         if not os.path.exists(os.path.join(self.buildpath, 'gpg')):
             os.mkdir(os.path.join(self.buildpath, 'gpg'))
-        gpgfile = os.path.join(self.buildpath, 'gpg',
-                               self.distopts['distribution'])
+        gpgfile = os.path.join(self.buildpath, 'gpg', distribution)
         if not os.path.exists(gpgfile):
             self.cmd = 'create'
-            self.w(_('Chroot must be created'), 2)
+            debug(_('%s chroot must be created') % distribution)
             return
         if self.distribution in self.runtime_option('alwaysupdate'):
             self.cmd = 'update'
-            self.w(_('Chroot must be updated'), 2)
+            debug(_('%s chroot must be updated') % distribution)
             return
         uri = '%s/dists/%s/Release.gpg' % (self.distopts['mirror'],
-                                           self.distopts['distribution'])
+                                           distribution)
         try:
             remote = urlopen(uri).read()
         except (HTTPError, URLError):
-            self.w(_('Unable to fetch %s') % uri)
-            self.cmd = update
-            self.w(_('Chroot must be updated'), 2)
+            error(_('Unable to fetch %s') % uri)
+            self.cmd = 'update'
+            debug(_('%s chroot must be updated') % distribution)
             return
         remote_sha = sha256()
         gpgfile_sha = sha256()
@@ -250,11 +264,11 @@ class Build:
                 gpgfile_sha.update(fd.read())
         except OSError:
             self.cmd = 'create'
-            self.w(_('Chroot must be created'), 2)
+            debug(_('%s chroot must be created') % distribution)
             return
         if remote_sha.digest() != gpgfile_sha.digest():
             self.cmd = 'update'
-            self.w(_('Chroot must be updated'), 2)
+            debug(_('%s chroot must be updated') % distribution)
 
     def parse_distribution_options(self):
         conf = {'components': ('[^#]?COMPONENTS="?(.*[^"])"?\n', 'COMPONENTS'),
@@ -271,11 +285,13 @@ class Build:
                 with open(self.packagepath, 'r') as fd:
                     data = fd.read()
             except IOError:
-                self.e(_('Unable to open %s') % self.packagepath)
+                error(_('Unable to open %s') % self.packagepath)
+                raise RuntimeError
             try:
                 distro = findall('Distribution:\s+(\S+)', data)[0]
             except IndexError:
-                self.e(_('Bad .changes file: %s') % self.packagepath)
+                error(_('Bad .changes file: %s') % self.packagepath)
+                raise RuntimeError
             self.distribution = distro.lower()
         self.map_distribution()
         self.buildpath = os.path.join(self.packagedir, self.distribution)
@@ -286,14 +302,16 @@ class Build:
                 data = fd.read()
         except IOError:
             self.remove_files()
-            self.log.e(_('Unable to open %s') % self.configfile)
-        for elem in conf.keys():
+            error(_('Unable to open %s') % self.configfile)
+            exit(2)
+        for elem in conf:
             try:
                 if not elem in self.distopts or not self.distopts[elem]:
                     self.distopts[elem] = findall(conf[elem][0], data)[0]
             except IndexError:
-                self.e(_('Please set %(parm)s in %s(conf)s') %
-                       {'parm': conf[elem][0], 'conf': conf})
+                error(_('Please set %(parm)s in %s(conf)s') %
+                      {'parm': conf[elem][0], 'conf': conf})
+                raise RuntimeError
         if self.origin:
             originconfig = os.path.join(self.opts.get('default', 'configdir'),
                                         self.origin)
@@ -301,61 +319,87 @@ class Build:
                 with open(originconfig) as fd:
                     data = fd.read()
             except IOError:
-                self.log.e(_('Unable to open %s') % originconfig)
+                error(_('Unable to open %s') % originconfig)
+                exit(2)
             for elem in ('origin', 'ocomponents'):
                 try:
                     self.distopts[elem] = findall(conf[elem][0], data)[0]
                 except IndexError:
-                    self.e(_('Please set %(parm)s in %s(conf)s') %
-                           {'parm': conf[elem][0], 'conf': conf})
+                    error(_('Please set %(parm)s in %s(conf)s') %
+                          {'parm': conf[elem][0], 'conf': conf})
+                    raise RuntimeError
         else:
             self.origin = self.distribution
 
     def prepare_pbuilder(self):
+        builder = self.opts.get('default', 'builder')
+        architecture = self.opts.get('default', 'architecture')
+        if architecture == 'system':
+            b_arch = check_output(['dpkg-architecture', '-qDEB_BUILD_ARCH'])
+            architecture = b_arch.strip().decode('utf-8')
+        debootstrap = self.opts.get('default', 'debootstrap')
+        mod = Module((self.opts, self.rtopts, self.conffile))
+        debug(_('Pre-chroot maintenance hooks launched'))
+        mod.execute_hook('pre_chroot', {'cfg': self.configfile,
+                                        'directory': self.buildpath,
+                                        'opts': self.opts,
+                                        'distribution': self.distribution,
+                                        'architecture': architecture,
+                                        'cmd': self.cmd})
         for d in ('aptcache', 'build', 'logs', 'pool'):
             if not os.path.exists(os.path.join(self.buildpath, d)):
                 os.mkdir(os.path.join(self.buildpath, d))
-        for f in ('Packages.gz', 'Release'):
-            repo_file = os.path.join(self.buildpath, 'pool', f)
-            if not os.path.exists(repo_file):
-                with open(os.path.splitext(repo_file)[0], 'w') as fd:
-                    pass
-        try:
-            call(['gzip', '-9', '-f', os.path.join(self.buildpath, 'pool',
-                                                   'Packages')], stderr=PIPE)
-        except OSError:
-            self.w(_('Unable to launch %s') % 'gzip')
-        builder = self.opts.get('default', 'builder')
         if builder == 'cowbuilder':
             base = '--basepath'
         else:
             base = '--basetgz'
         with open(os.devnull, 'w') as fd:
             try:
-                self.w(_('Launching %(builder)s %(cmd)s')
-                       % {'builder': builder, 'cmd': self.cmd} , 2)
+                debug(_('Launching %(builder)s %(cmd)s')
+                      % {'builder': builder, 'cmd': self.cmd})
                 if call([builder, '--%s' % self.cmd, '--override-config',
                         base, '%s/%s' % (self.buildpath, self.distribution),
                         '--buildplace', '%s/build' % self.buildpath,
                         '--aptcache', '%s/aptcache' % self.buildpath,
+                        '--architecture', architecture,
                         '--logfile', '%s/logs/%s.%s' %
                         (self.buildpath, self.cmd, strftime('%Y%m%d_%H%M%S')),
-                        '--configfile', '%s' % self.configfile],
+                        '--configfile', '%s' % self.configfile,
+                        '--debootstrap', debootstrap],
                         stdout=fd, stderr=fd):
-                    self.release_lock()
-                    self.e(_('%(builder)s %(cmd)s failed') %
-                           {'builder': builder, 'cmd': self.cmd})
+                    debug(_('Post-chroot maintenance hooks launched'))
+                    mod.execute_hook('post_chroot', {'cfg': self.configfile,
+                                     'directory': self.buildpath,
+                                     'opts': self.opts,
+                                     'distribution': self.distribution,
+                                     'architecture': architecture,
+                                     'cmd': self.cmd, 'success': False})
+                    error(_('%(builder)s %(cmd)s failed') %
+                          {'builder': builder, 'cmd': self.cmd})
+                    raise RuntimeError
             except OSError:
-                self.release_lock()
-                self.e(_('Unable to launch %s') % builder)
-
-    def release_lock(self):
-        self.lockfile.release()
+                debug(_('Post-chroot maintenance hooks launched'))
+                mod.execute_hook('post_chroot', {'cfg': self.configfile,
+                                 'directory': self.buildpath,
+                                 'opts': self.opts,
+                                 'distribution': self.distribution,
+                                 'architecture': architecture,
+                                 'cmd': self.cmd, 'success': False})
+                error(_('Unable to launch %s') % builder)
+                raise RuntimeError
+        debug(_('Post-chroot maintenance hooks launched'))
+        mod.execute_hook('post_chroot', {'cfg': self.configfile,
+                                         'directory': self.buildpath,
+                                         'opts': self.opts,
+                                         'distribution': self.distribution,
+                                         'architecture': architecture,
+                                         'cmd': self.cmd, 'success': True})
 
     def remove_files(self):
         for pkgfile in self.files:
             if os.path.exists(pkgfile):
                 os.remove(pkgfile)
+                debug(_('File %s removed') % pkgfile)
 
     def runtime_option(self, option):
         self.rtopts.read(self.conffile)
@@ -367,7 +411,6 @@ class Build:
     def setup_pbuilder(self):
         if not os.path.exists(os.path.join(self.buildpath)):
             os.mkdir(os.path.join(self.buildpath))
-        self.acquire_lock()
         self.needs_update()
         if self.cmd:
             self.prepare_pbuilder()
@@ -378,11 +421,10 @@ class Build:
             try:
                 remote = urlopen(uri).read()
             except (HTTPError, URLError):
-                self.release_lock()
-                self.e(_('Unable to fetch %s') % uri)
-            with open(gpgfile, 'w') as fd:
+                error(_('Unable to fetch %s') % uri)
+                raise RuntimeError
+            with open(gpgfile, 'wb') as fd:
                 fd.write(remote)
-        self.release_lock()
 
 
 class FullBuild(Build):
@@ -391,22 +433,30 @@ class FullBuild(Build):
         self.full = True
         self.packagepath = os.path.join(self.packagedir, self.package)
         self.files.add(self.packagepath)
-        self.w(_('Processing %s') % self.package)
+        debug(_('File %s added') % self.packagepath)
+        info(_('Processing %s') % self.package)
         try:
             with open(self.packagepath, 'r') as fd:
                 data = fd.read()
         except IOError:
-            self.e(_('Unable to open %s') % self.packagepath)
+            error(_('Unable to open %s') % self.packagepath)
+            raise RuntimeError
         try:
             for entry in findall('\s\w{32}\s\d+\s\S+\s\S+\s(.*)', data):
-                self.files.add(os.path.join(self.packagedir, entry))
+                entry = os.path.join(self.packagedir, entry)
+                self.files.add(entry)
+                debug(_('File %s added') % entry)
         except IndexError:
-            self.e(_('Bad .changes file: %s') % self.packagepath)
-        gpg = GPG(self.opts, self.packagepath)
-        if gpg.gpg:
-            if gpg.sig:
-                self.uploader = gpg.sig
-            else:
-                self.remove_files()
-                self.e(gpg.error)
-        self.build()
+            error(_('Bad .changes file: %s') % self.packagepath)
+            raise RuntimeError
+        try:
+            with GPG(self.opts, self.packagepath) as gpg:
+                try:
+                    self.uploader = gpg.check()
+                    self.build()
+                except RuntimeError:
+                    self.remove_files()
+                    error(gpg.error())
+                    raise RuntimeError
+        except IOError:
+            pass
