@@ -1,7 +1,7 @@
 # Deb-o-Matic
 #
 # Copyright (C) 2008-2009 David Futcher
-# Copyright (C) 2012-2013 Luca Falavigna
+# Copyright (C) 2012-2014 Luca Falavigna
 #
 # Author: David Futcher <bobbo@ubuntu.com>
 #
@@ -19,6 +19,7 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
 import os
+from logging import debug
 from sys import path
 
 
@@ -27,7 +28,7 @@ class Module():
     def __init__(self, opts):
         (self.opts, self.rtopts, self.conffile) = opts
         self.use_modules = True
-        self.modules_list = []
+        self.modules_list = set()
         if not self.opts.has_option('modules', 'modules'):
             self.use_modules = False
         elif self.opts.get('modules', 'modules') == "0":
@@ -38,24 +39,27 @@ class Module():
         else:
             self.mod_path = ''
         try:
-            self.modules_list = os.listdir(self.mod_path)
+            self.modules_list = set(os.listdir(self.mod_path))
         except OSError:
             self.use_modules = False
-        if len(self.modules_list) == 0:
+        if not self.modules_list:
             self.use_modules = False
         if self.use_modules:
             self.instances = {}
             for module in self.modules_list:
+                if module.endswith('.pyc'):
+                    continue
                 module_split = module.split(".")[:-1]
                 module = ""
                 for i in module_split:
                     module += i
                 try:
-                    exec 'from %s import DebomaticModule_%s' % \
-                         (module, module)
-                    exec 'self.instances["%s"] = DebomaticModule_%s()' % \
-                         (module, module)
-                except NameError:
+                    exec('from %s import DebomaticModule_%s' %
+                         (module, module))
+                    exec('self.instances["%s"] = DebomaticModule_%s()' %
+                         (module, module))
+                    debug(_('Module %s loaded') % module)
+                except (NameError, SyntaxError):
                     pass
 
     def execute_hook(self, hook, args):
@@ -66,11 +70,13 @@ class Module():
                 blist_mods = blist_mods.split()
             else:
                 blist_mods = []
-            modules = [ i for i in self.instances.keys()
-                       if i not in blist_mods ]
+            modules = [i for i in self.instances
+                       if i not in blist_mods]
             modules.sort()
             for module in modules:
                 try:
-                    exec 'self.instances["%s"].%s(args)' % (module, hook)
+                    debug(_('Executing hook %(hook)s from module %(mod)s') %
+                          {'hook': hook, 'mod': module})
+                    exec('self.instances["%s"].%s(args)' % (module, hook))
                 except AttributeError:
                     pass
