@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012-2013 Luca Falavigna
+# Copyright (C) 2012-2014 Luca Falavigna
 #
 # Author: Luca Falavigna <dktrkranz@debian.org>
 #
@@ -19,18 +19,15 @@
 #
 # Deb-o-Matic documentation build configuration file
 
-import os
-import sys
-
 
 extensions = []
 templates_path = ['_templates']
 source_suffix = '.rst'
 master_doc = 'index'
-project = u'Deb-o-Matic'
-copyright = u'2007-2013, Luca Falavigna'
-version = '0.11'
-release = '0.11'
+project = 'Deb-o-Matic'
+copyright = '2007-2014, Luca Falavigna'
+version = '0.12'
+release = '0.12'
 exclude_patterns = ['_build']
 pygments_style = 'sphinx'
 html_theme = 'default'
@@ -38,11 +35,11 @@ html_static_path = ['_static']
 html_use_index = True
 htmlhelp_basename = 'Deb-o-Maticdoc'
 latex_documents = [
-  ('index', 'Deb-o-Matic.tex', u'Deb-o-Matic Documentation',
-   u'Luca Falavigna', 'manual')]
+    ('index', 'Deb-o-Matic.tex', 'Deb-o-Matic Documentation',
+     'Luca Falavigna', 'manual')]
 latex_elements = {
-  'classoptions': ',oneside',
-  'babel': '\\usepackage[english]{babel}'}
+    'classoptions': ',oneside',
+    'babel': '\\usepackage[english]{babel}'}
 man_pages = [
-    ('index', 'deb-o-matic', u'Deb-o-Matic Documentation',
-     [u'Luca Falavigna'], 1)]
+    ('index', 'deb-o-matic', 'Deb-o-Matic Documentation',
+     ['Luca Falavigna'], 1)]
