@@ -1,7 +1,7 @@
 # Deb-o-Matic - Contents module
 #
 # Copyright (C) 2009 Alessio Treglia
-# Copyright (C) 2010-2013 Luca Falavigna
+# Copyright (C) 2010-2014 Luca Falavigna
 #
 # Authors: Alessio Treglia <quadrispro@ubuntu.com>
 #          Luca Falavigna <dktrkranz@debian.org>
@@ -40,4 +40,5 @@ class DebomaticModule_Contents:
                 break
         if changes_file:
             with open(contents_file, 'w') as fd:
-                call([self.debc, changes_file], stdout=fd)
+                call([self.debc, '-a%s' % args['architecture'], changes_file],
+                     stdout=fd)
